import numpy as np
from sklearn.linear_model import Lasso, LassoLars
from sklearn.model_selection import cross_validate
from sklearn.metrics import r2_score, mean_absolute_error


def cross_validation_scheme(descriptor_arr, target_arr, n_cv, estimator=None):
    """
    Perform n-fold validation scheme with given regression estimator.
    Parameters
    ----------
    descriptor_arr : numpy.array
                     2-D array containing m samples and n descriptors
    target_arr : numpy.array
                 1-D array containing target property to predict.
    n_cv : int
           Number of folds used in cross validation scheme.
    estimator :  object
                 Estimator object for regression algorithm to use.
    Returns
    -------
    cvs :   object
            The whole cross validation scheme, returned as an object.
    """
    if estimator is None:
        # Use LassoLars as default estimator
        estimator = LassoLars(alpha=1.5e-4,max_iter=100000)

    # Config the estimator and cv scheme
    print(str(n_cv) + '-fold cross validation ongoing ...')
    estimator.fit(descriptor_arr, target_arr)
    cvs = cross_validate(estimator, descriptor_arr, target_arr, cv=n_cv,
                         scoring=('r2', 'neg_mean_absolute_error'),
                         return_estimator=True,
                         return_train_score=True)
    # Compute and print major stat for the cv scheme
    mae_cv_train = -1 * cvs['train_neg_mean_absolute_error'].mean()
    mae_cv_test = -1 * cvs['test_neg_mean_absolute_error'].mean()
    r2_cv_train = cvs['train_r2'].mean()
    r2_cv_test = cvs['test_r2'].mean()
    print('\n')
    print('train R2: %.3f' % r2_cv_train)
    print('train MAE: %.2f nm' % mae_cv_train)
    print('test R2: %.3f' % r2_cv_test)
    print('test MAE: %.2f nm' % mae_cv_test)
    print('\n')
    print('Cross validation completes.')
    # return the cross validation scheme as a dictionary
    return cvs


def print_bc_and_lambda_stat(bc_arr, lambda_arr, bc_str='bc', lambda_str='lambda'):
    """
    Print dimension of input bc and major statistics for reference lambda values.
    ----------
    coord_arr : numpy.array
                         2-D array containing m samples and 3n_atom components.
    lambda_arr : numpy.array
                     1-D array containing reference lambda values.
    coord_str : str
                string to describe the coordinate array.
    lambda_str :  str
                   string to describe the lambda array.
    """
    print('\n')
    print('Dimension of ' + coord_str + ': ', coord_arr.shape)
    print('Dimension of ' + lambda_str + ': ', lambda_arr.shape[0])
    print('Max ' + lambda_str + ': %.2f nm' % np.max(lambda_arr))
    print('Min ' + lambda_str + ': %.2f nm' % np.min(lambda_arr))
    print('Avg ' + lambda_str + ': %.2f nm' % np.mean(lambda_arr))
    print('std ' + lambda_str + ': %.2f nm' % np.std(lambda_arr))
    print('\n')


def print_prediction_summary(target_arr, predictions_arr):
    """
    Print prediction summary for randomly chosen subset .
    ----------
    target_arr : numpy.array
                 1-D array containing target property to predict.
    predictions_arr : numpy.array
                 1-D array containing predictions used randomly chosen subset as input.
    """
    print('\n')
    print('Prediction R2: %.3f' % r2_score(target_arr, predictions_arr))
    print('Prediction MAE: %.2f nm' % mean_absolute_error(target_arr, predictions_arr))
